@@ -22,15 +22,13 @@ __all__ = [
 
 
 import os
+import atexit
 
-# BAW 2013-04-23: If we need something more sophisticated, lazr.config is the
-# way to go.  It provides a nice testing infrastruction (pushing/popping of
-# configurations), schema definition, and a slew of useful data type
-# conversion functions.  For now, we limit the non-stdlib dependencies and
-# roll our own.
 from configparser import ConfigParser
+from contextlib import ExitStack
 from pkg_resources import resource_filename
-from systemimage.helpers import Bag, as_object, as_timedelta, as_loglevel
+from systemimage.helpers import (
+    Bag, as_loglevel, as_object, as_timedelta, makedirs, temporary_directory)
 
 
 def expand_path(path):
@@ -44,10 +42,21 @@ class Configuration:
         ini_path = resource_filename('systemimage.data', 'client.ini')
         self.load(ini_path)
         self._override = False
+        # 2013-10-14 BAW This is a placeholder for rendezvous between the
+        # downloader and the D-Bus service.  When running udner D-Bus and we
+        # get a `paused` signal from the download manager, we need this to
+        # plumb through an UpdatePaused signal to our clients.  It rather
+        # sucks that we need a global for this, but I can't get the plumbing
+        # to work otherwise.  This seems like the least horrible place to
+        # stash this global.
+        self.dbus_service = None
         # Cache/overrides.
         self._device = None
         self._build_number = None
         self._channel = None
+        self._tempdir = None
+        self._resources = ExitStack()
+        atexit.register(self._resources.close)
 
     def load(self, path, *, override=False):
         parser = ConfigParser()
@@ -76,7 +85,6 @@ class Configuration:
         if override:
             return
         self.system = Bag(converters=dict(timeout=as_timedelta,
-                                          threads=int,
                                           build_file=expand_path,
                                           loglevel=as_loglevel,
                                           settings_db=expand_path,
@@ -141,6 +149,15 @@ class Configuration:
     def channel(self, value):
         self._channel = value
 
+    @property
+    def tempdir(self):
+        if self._tempdir is None:
+            makedirs(self.system.tempdir)
+            self._tempdir = self._resources.enter_context(
+                temporary_directory(prefix='system-image-',
+                                    dir=self.system.tempdir))
+        return self._tempdir
+
 
 # Define the global configuration object.  Normal use can be as simple as:
 #
