@@ -22,7 +22,6 @@ __all__ = [
 
 
 import os
-#import pickle
 import shutil
 import hashlib
 import logging
@@ -36,7 +35,7 @@ from systemimage.channel import Channels
 from systemimage.config import config
 from systemimage.download import DBusDownloadManager
 from systemimage.gpg import Context, SignatureError
-from systemimage.helpers import makedirs
+from systemimage.helpers import makedirs, safe_remove
 from systemimage.index import Index
 from systemimage.keyring import KeyringError, get_keyring
 from urllib.parse import urljoin
@@ -51,6 +50,12 @@ class ChecksumError(Exception):
     """Exception raised when a file's checksum does not match."""
 
 
+def _copy_if_missing(src, dstdir):
+    dst_path = os.path.join(dstdir, os.path.basename(src))
+    if os.path.exists(src) and not os.path.exists(dst_path):
+        shutil.copy(src, dstdir)
+
+
 class State:
     def __init__(self, candidate_filter=None):
         # Variables which manage state transitions.
@@ -64,18 +69,7 @@ class State:
         self.winner = None
         self.files = []
         self.downloader = DBusDownloadManager()
-        # See if there is a state file to unpickle.
-        first_step = self._get_blacklist_1
-        ## try:
-        ##     with open(config.system.state_file, 'rb') as fp:
-        ##         self.blacklist = pickle.load(fp)
-        ##         self.winner = pickle.load(fp)
-        ##         pickle_date = pickle.load(fp)
-        ##         # XXX check for stale file.
-        ##     first_step = self._download_files
-        ## except FileNotFoundError:
-        ##     pass
-        self._next.append(first_step)
+        self._next.append(self._cleanup)
 
     def __iter__(self):
         return self
@@ -112,7 +106,11 @@ class State:
             except (StopIteration, IndexError):
                 # We're done.
                 break
-            step()
+            try:
+                step()
+            except:
+                log.exception('uncaught exception in state machine')
+                raise
             self._debug_step += 1
             if name[1:] == stop_after:
                 break
@@ -136,9 +134,38 @@ class State:
                 # skip this step.
                 self._next.appendleft(step)
                 break
-            step()
+            try:
+                step()
+            except:
+                log.exception('uncaught exception in state machine')
+                raise
             self._debug_step += 1
 
+    def _cleanup(self):
+        """Clean up the destination directories.
+
+        Removes all residual files from the cache and data partitions except
+        `log` and `last_log` in the cache partition.
+        """
+        cache_dir = config.updater.cache_partition
+        data_dir = config.updater.data_partition
+        try:
+            files = os.listdir(cache_dir)
+        except FileNotFoundError:
+            pass
+        else:
+            for filename in files:
+                if filename in ('log', 'last_log'):
+                    continue
+                safe_remove(os.path.join(cache_dir, filename))
+        # Remove only the blacklist files (and generic keyring files) since
+        # these are the only ones that will be downloaded to this location.
+        safe_remove(os.path.join(data_dir, 'blacklist.tar.xz'))
+        safe_remove(os.path.join(data_dir, 'blacklist.tar.xz.asc'))
+        safe_remove(os.path.join(data_dir, 'keyring.tar.xz'))
+        safe_remove(os.path.join(data_dir, 'keyring.tar.xz.asc'))
+        self._next.append(self._get_blacklist_1)
+
     def _get_blacklist_1(self):
         """First try to get the blacklist."""
         # If there is no image master key, download one now.  Don't worry if
@@ -168,11 +195,10 @@ class State:
             # There is no blacklist.
             log.info('No blacklist found')
         else:
-            # Move the keyring.gpg file to a safe place inside our temporary
-            # directory.  It is the responsibility of the code running the
-            # state machine to clear out the temporary directory.
+            # After successful download, the blacklist.tar.xz will be living
+            # in the data partition.
             self.blacklist = os.path.join(
-                config.system.tempdir, 'blacklist.tar.xz')
+                config.updater.data_partition, 'blacklist.tar.xz')
             log.info('Local blacklist file: {}', self.blacklist)
         self._next.append(self._get_channel)
 
@@ -195,11 +221,10 @@ class State:
         except FileNotFoundError:
             log.info('No blacklist found on second attempt')
         else:
-            # Move the keyring.gpg file to a safe place inside our temporary
-            # directory.  It is the responsibility of the code running the
-            # state machine to clear out the temporary directory.
+            # After successful download, the blacklist.tar.xz will be living
+            # in the data partition.
             self.blacklist = os.path.join(
-                config.system.tempdir, 'blacklist.tar.xz')
+                config.updater.data_partition, 'blacklist.tar.xz')
             log.info('Local blacklist file: {}', self.blacklist)
         self._next.append(self._get_channel)
 
@@ -214,9 +239,9 @@ class State:
             get_keyring(
                 'image-signing', 'gpg/image-signing.tar.xz', 'image-master')
         channels_url = urljoin(config.service.https_base, 'channels.json')
-        channels_path = os.path.join(config.system.tempdir, 'channels.json')
+        channels_path = os.path.join(config.tempdir, 'channels.json')
         asc_url = urljoin(config.service.https_base, 'channels.json.asc')
-        asc_path = os.path.join(config.system.tempdir, 'channels.json.asc')
+        asc_path = os.path.join(config.tempdir, 'channels.json.asc')
         log.info('Looking for: {}', channels_url)
         with ExitStack() as stack:
             self.downloader.get_files([
@@ -313,7 +338,7 @@ class State:
         """Get and verify the index.json file."""
         index_url = urljoin(config.service.https_base, index)
         asc_url = index_url + '.asc'
-        index_path = os.path.join(config.system.tempdir, 'index.json')
+        index_path = os.path.join(config.tempdir, 'index.json')
         asc_path = index_path + '.asc'
         with ExitStack() as stack:
             self.downloader.get_files([
@@ -365,22 +390,10 @@ class State:
         if len(self.winner) > 0:
             winning_path = [str(image.version) for image in self.winner]
             log.info('Upgrade path is {}'.format(COLON.join(winning_path)))
-            self._next.append(self._persist)
+            self._next.append(self._download_files)
         else:
             log.info('Already up-to-date')
 
-    def _persist(self):
-        """Persist enough state to resume right to downloading files."""
-        ## with open(config.system.state_file, 'wb') as fp:
-        ##     # Things we need to pickle:
-        ##     # - the location of our blacklist file
-        ##     # - the set of calculated winners
-        ##     # - the current datetime
-        ##     pickle.dump(self.blacklist, fp)
-        ##     pickle.dump(self.winner, fp)
-        ##     pickle.dump(datetime.now(), fp)
-        self._next.append(self._download_files)
-
     def _download_files(self):
         """Download and verify all the winning upgrade path's files."""
         downloads = []
@@ -391,7 +404,7 @@ class State:
             # Re-pack for arguments to get_files() and to collate the size,
             # signature path, and checksum for the downloadable file.
             dst = os.path.join(
-                config.system.tempdir, os.path.basename(filerec.path))
+                config.updater.cache_partition, os.path.basename(filerec.path))
             downloads.append((
                 urljoin(config.service.http_base, filerec.path),
                 dst,
@@ -400,7 +413,8 @@ class State:
             self.files.append((dst, (image_number, filerec.order)))
             # Add the signature file, and associate the two.
             asc = os.path.join(
-                config.system.tempdir, os.path.basename(filerec.signature))
+                config.updater.cache_partition,
+                os.path.basename(filerec.signature))
             downloads.append((
                 urljoin(config.service.http_base, filerec.signature),
                 asc))
@@ -414,8 +428,13 @@ class State:
         keyrings = [config.gpg.image_signing]
         if os.path.exists(config.gpg.device_signing):
             keyrings.append(config.gpg.device_signing)
+        # We must make sure that none of the destination file paths exist,
+        # otherwise the downloader will throw exceptions.
+        for url, dst in downloads:
+            safe_remove(dst)
         # Now, download all the files, providing logging feedback on progress.
-        self.downloader.get_files(downloads)
+        # This download can be paused.
+        self.downloader.get_files(downloads, pausable=True)
         with ExitStack() as stack:
             # Set things up to remove the files if a SignatureError gets
             # raised or if the checksums don't match.  If everything's okay,
@@ -440,7 +459,7 @@ class State:
                         raise ChecksumError(dst, got, checksum)
             # Everything is fine so nothing needs to be cleared.
             stack.pop_all()
-        log.info('all files available in {}', config.system.tempdir)
+        log.info('all files available in {}', config.updater.cache_partition)
         # Now, copy the files from the temporary directory into the location
         # for the upgrader.
         self._next.append(self._move_files)
@@ -448,28 +467,19 @@ class State:
     def _move_files(self):
         # The upgrader already has the archive-master, so we don't need to
         # copy it.  The image-master, image-signing, and device-signing (if
-        # there is one) keys go to the cache partition.
+        # there is one) keys go to the cache partition.  They may already be
+        # there if they had to be downloaded, but if not, they're in /var/lib
+        # and now need to be copied to the cache partition.  The blacklist
+        # keyring, if there is one, should already exist in the data partition.
         cache_dir = config.updater.cache_partition
-        data_dir = config.updater.data_partition
         makedirs(cache_dir)
-        makedirs(data_dir)
         # Copy the keyring.tar.xz and .asc files.
-        shutil.copy(config.gpg.image_master, cache_dir)
-        shutil.copy(config.gpg.image_master + '.asc', cache_dir)
-        shutil.copy(config.gpg.image_signing, cache_dir)
-        shutil.copy(config.gpg.image_signing + '.asc', cache_dir)
-        if os.path.exists(config.gpg.device_signing):
-            shutil.copy(config.gpg.device_signing, cache_dir)
-            shutil.copy(config.gpg.device_signing + '.asc', cache_dir)
-        # The blacklist file (if there is one) gets copied to the data
-        # partition.
-        if self.blacklist is not None:
-            shutil.copy(self.blacklist, data_dir)
-            shutil.copy(self.blacklist + '.asc', data_dir)
-        # Now move all the downloaded data files to the cache.
-        for src, (image_number, order) in self.files:
-            dst = os.path.join(cache_dir, os.path.basename(src))
-            shutil.move(src, dst)
+        _copy_if_missing(config.gpg.image_master, cache_dir)
+        _copy_if_missing(config.gpg.image_master + '.asc', cache_dir)
+        _copy_if_missing(config.gpg.image_signing, cache_dir)
+        _copy_if_missing(config.gpg.image_signing + '.asc', cache_dir)
+        _copy_if_missing(config.gpg.device_signing, cache_dir)
+        _copy_if_missing(config.gpg.device_signing + '.asc', cache_dir)
         # Issue the reboot.
         self._next.append(self._prepare_recovery)
 
