@@ -179,12 +179,12 @@ def make_http_server(directory, port, certpem=None, keypem=None):
 def configuration(function):
     """Decorator that produces a temporary configuration for testing.
 
-    The config_00.ini template is copied to a temporary file and the
-    [system]tempdir variable is filled in with the location for a, er,
-    temporary temporary directory.  This temporary configuration file is
-    loaded up and the global configuration object is patched so that all
-    other code will see it instead of the default global configuration
-    object.
+    The config_00.ini template is copied to a temporary file and the the
+    various file system locations are filled in with the location for a,
+    er, temporary temporary directory.  This temporary configuration
+    file is loaded up and the global configuration object is patched so
+    that all other code will see it instead of the default global
+    configuration object.
 
     Everything is properly cleaned up after the test method exits.
     """
@@ -205,10 +205,12 @@ def configuration(function):
             stack.enter_context(patch('systemimage.config._config', config))
             stack.enter_context(patch('systemimage.device.check_output',
                                       return_value='nexus7'))
-            # 2013-07-23 BAW: Okay, this is wicked.  If the test method takes
-            # an 'ini_file' argument, pass the temporary ini file path to it
-            # as a keyword argument.  Mostly I do this so that I don't have to
-            # change the signature of every existing test method.
+            # Make sure the cache_partition and data_partition exist.
+            makedirs(config.updater.cache_partition)
+            makedirs(config.updater.data_partition)
+            # The method under test is allowed to specify some additional
+            # keyword arguments, in order to pass some variables in from the
+            # wrapper.
             signature = inspect.signature(function)
             if 'ini_file' in signature.parameters:
                 kws['ini_file'] = ini_file
@@ -312,8 +314,8 @@ def setup_keyrings(*keyrings, use_config=None):
             raise AssertionError('unknown key type: {}'.format(keyring))
         # The local keyrings live in the .gpg file with the same keyring name
         # as the .tar.xz file, but cached in the temporary directory.
-        copy(keyring + '.gpg', (config.system.tempdir if use_config is None
-                                else use_config.system.tempdir))
+        copy(keyring + '.gpg', (config.tempdir if use_config is None
+                                else use_config.tempdir))
         # Now set up the .tar.xz and .tar.xz.asc files in the destination.
         json_data = dict(type=keyring)
         dst = getattr((config.gpg if use_config is None
