@@ -30,6 +30,8 @@ import random
 import unittest
 
 from contextlib import ExitStack
+from datetime import datetime, timedelta
+from gi.repository import GLib
 from systemimage.config import config
 from systemimage.download import Canceled, DBusDownloadManager
 from systemimage.helpers import temporary_directory
@@ -45,14 +47,14 @@ MiB = 1024 * 1024
 def _http_pathify(downloads):
     return [
         (urljoin(config.service.http_base, url),
-         os.path.join(config.system.tempdir, filename)
+         os.path.join(config.tempdir, filename)
         ) for url, filename in downloads]
 
 
 def _https_pathify(downloads):
     return [
         (urljoin(config.service.https_base, url),
-         os.path.join(config.system.tempdir, filename)
+         os.path.join(config.tempdir, filename)
         ) for url, filename in downloads]
 
 
@@ -83,7 +85,7 @@ class TestDownloads(unittest.TestCase):
             ('index_01.json', 'index.json'),
             ]))
         self.assertEqual(
-            set(os.listdir(config.system.tempdir)),
+            set(os.listdir(config.tempdir)),
             set(['channels.json', 'index.json']))
 
     @configuration
@@ -97,7 +99,7 @@ class TestDownloads(unittest.TestCase):
         DBusDownloadManager().get_files(_http_pathify([
             ('user-agent.txt', 'user-agent.txt'),
             ]))
-        path = os.path.join(config.system.tempdir, 'user-agent.txt')
+        path = os.path.join(config.tempdir, 'user-agent.txt')
         with open(path, 'r', encoding='utf-8') as fp:
             user_agent = fp.read()
         self.assertEqual(
@@ -119,7 +121,7 @@ class TestDownloads(unittest.TestCase):
             ('index_01.json', 'index.json'),
             ]))
         self.assertEqual(
-            set(os.listdir(config.system.tempdir)),
+            set(os.listdir(config.tempdir)),
             set(['channels.json', 'index.json']))
         self.assertEqual(received_bytes, 555)
         self.assertEqual(total_bytes, 555)
@@ -144,7 +146,7 @@ class TestHTTPSDownloads(unittest.TestCase):
                 ('channels_01.json', 'channels.json'),
                 ]))
             self.assertEqual(
-                set(os.listdir(config.system.tempdir)),
+                set(os.listdir(config.tempdir)),
                 set(['channels.json']))
 
 
@@ -231,7 +233,7 @@ class TestDownloadBigFiles(unittest.TestCase):
     @configuration
     def test_cancel(self):
         # Try to cancel the download of a big file.
-        self.assertEqual(os.listdir(config.system.tempdir), [])
+        self.assertEqual(os.listdir(config.tempdir), [])
         with ExitStack() as stack:
             serverdir = stack.enter_context(temporary_directory())
             stack.push(make_http_server(serverdir, 8980))
@@ -255,14 +257,14 @@ class TestDownloadBigFiles(unittest.TestCase):
                     ('bigfile_1.dat', 'bigfile_1.dat'),
                     ('bigfile_2.dat', 'bigfile_2.dat'),
                     ]))
-            self.assertEqual(os.listdir(config.system.tempdir), [])
+            self.assertEqual(os.listdir(config.tempdir), [])
 
     @configuration
     def test_download_404(self):
         # Start a group download of some big files.   One of the files won't
         # exist, so the entire group download should fail, and none of the
         # files should exist in the destination.
-        self.assertEqual(os.listdir(config.system.tempdir), [])
+        self.assertEqual(os.listdir(config.tempdir), [])
         with ExitStack() as stack:
             serverdir = stack.enter_context(temporary_directory())
             stack.push(make_http_server(serverdir, 8980))
@@ -283,4 +285,49 @@ class TestDownloadBigFiles(unittest.TestCase):
                               DBusDownloadManager().get_files,
                               downloads)
             # The temporary directory is empty.
-            self.assertEqual(os.listdir(config.system.tempdir), [])
+            self.assertEqual(os.listdir(config.tempdir), [])
+
+    @configuration
+    def test_download_pause_resume(self):
+        with ExitStack() as stack:
+            serverdir = stack.enter_context(temporary_directory())
+            stack.push(make_http_server(serverdir, 8980))
+            # Create a couple of big files to download.
+            with open(os.path.join(serverdir, 'bigfile_1.dat'), 'wb') as fp:
+                fp.write(b'x' * 10 * MiB)
+            with open(os.path.join(serverdir, 'bigfile_2.dat'), 'wb') as fp:
+                fp.write(b'x' * 10 * MiB)
+            with open(os.path.join(serverdir, 'bigfile_3.dat'), 'wb') as fp:
+                fp.write(b'x' * 10 * MiB)
+            downloads = _http_pathify([
+                ('bigfile_1.dat', 'bigfile_1.dat'),
+                ('bigfile_2.dat', 'bigfile_2.dat'),
+                ('bigfile_3.dat', 'bigfile_3.dat'),
+                ])
+            downloader = DBusDownloadManager()
+            pauses = []
+            def do_paused(self, signal, path, paused):
+                if paused:
+                    pauses.append(datetime.now())
+            resumes = []
+            def do_resumed(self, signal, path, resumed):
+                if resumed:
+                    resumes.append(datetime.now())
+            def pause_on_start(self, signal, path, started):
+                if started:
+                    downloader.pause()
+                    GLib.timeout_add_seconds(3, downloader.resume)
+            stack.enter_context(
+                patch('systemimage.download.DownloadReactor._do_paused',
+                      do_paused))
+            stack.enter_context(
+                patch('systemimage.download.DownloadReactor._do_resumed',
+                      do_resumed))
+            stack.enter_context(
+                patch('systemimage.download.DownloadReactor._do_started',
+                      pause_on_start))
+            downloader.get_files(downloads, pausable=True)
+            self.assertEqual(len(pauses), 1)
+            self.assertEqual(len(resumes), 1)
+            self.assertGreaterEqual(resumes[0] - pauses[0],
+                                    timedelta(seconds=2.5))
