@@ -31,10 +31,12 @@ __all__ = [
 
 import os
 import hashlib
+import tempfile
 import unittest
 
 from contextlib import ExitStack
 from datetime import datetime, timezone
+from functools import partial
 from subprocess import CalledProcessError
 from systemimage.config import config
 from systemimage.gpg import SignatureError
@@ -73,6 +75,67 @@ class TestState(unittest.TestCase):
     def tearDown(self):
         self._stack.close()
 
+    @configuration
+    def test_cleanup(self):
+        # All residual files from the cache and data partitions are removed,
+        # except `log` and `last_log` in the cache partition.
+        wopen = partial(open, mode='w', encoding='utf-8')
+        cache_partition = config.updater.cache_partition
+        data_partition = config.updater.data_partition
+        for i in range(10):
+            fd, path = tempfile.mkstemp(dir=cache_partition)
+            os.close(fd)
+            with wopen(path) as fp:
+                print('stale', file=fp)
+            fd, path = tempfile.mkstemp(dir=data_partition)
+            os.close(fd)
+            with wopen(path) as fp:
+                print('stale', file=fp)
+        with wopen(os.path.join(cache_partition, 'log')) as fp:
+            print('logger keeper', file=fp)
+        with wopen(os.path.join(cache_partition, 'last_log')) as fp:
+            print('logger keeper', file=fp)
+        with wopen(os.path.join(data_partition, 'log')) as fp:
+            print('stale log', file=fp)
+        with wopen(os.path.join(data_partition, 'last_log')) as fp:
+            print('stale log', file=fp)
+        with wopen(os.path.join(data_partition, 'blacklist.tar.xz')) as fp:
+            print('black list', file=fp)
+        with wopen(os.path.join(data_partition, 'blacklist.tar.xz.asc')) as fp:
+            print('black list', file=fp)
+        # Here are all the files before we start up the state machine.
+        self.assertEqual(len(os.listdir(cache_partition)), 12)
+        self.assertEqual(len(os.listdir(data_partition)), 14)
+        # Clean up step.
+        State().run_thru('cleanup')
+        # Two files in the cache partition are preserved.
+        self.assertEqual(set(os.listdir(cache_partition)),
+                         set(['log', 'last_log']))
+        # Only the blacklist files are removed from the data partition.
+        contents = os.listdir(data_partition)
+        self.assertEqual(len(contents), 12)
+        self.assertFalse('blacklist.tar.xz' in contents)
+        self.assertFalse('blacklist.tar.xz.asc' in contents)
+
+    @configuration
+    def test_cleanup_no_partition(self):
+        # If one or more of the partitions doesn't exist, no big deal.
+        #
+        # The cache partition doesn't exist.
+        os.rename(config.updater.cache_partition,
+                  config.updater.cache_partition + '.aside')
+        State().run_thru('cleanup')
+        # The data partition doesn't exist.
+        os.rename(config.updater.cache_partition + '.aside',
+                  config.updater.cache_partition)
+        os.rename(config.updater.data_partition,
+                  config.updater.data_partition + '.aside')
+        State().run_thru('cleanup')
+        # Neither partitions exist.
+        os.rename(config.updater.cache_partition,
+                  config.updater.cache_partition + '.aside')
+        State().run_thru('cleanup')
+
     @configuration
     def test_first_signature_fails_get_new_image_signing_key(self):
         # The first time we check the channels.json file, the signature fails,
@@ -167,22 +230,23 @@ class TestState(unittest.TestCase):
         setup_keyring_txz(
             'spare.gpg', 'archive-master.gpg', dict(type='image-master'),
             os.path.join(self._serverdir, 'gpg', 'image-master.tar.xz'))
-        # Run the state machine once to grab the blacklist.  This should fail
-        # with a signature error (internally).  There will be no blacklist.
+        # Run the state machine long enough to grab the blacklist.  This
+        # should fail with a signature error (internally).  There will be no
+        # blacklist.
         state = State()
-        next(state)
+        state.run_thru('get_blacklist_1')
         self.assertIsNone(state.blacklist)
         # Just to prove that the system image master key is going to change,
         # let's calculate the current one's checksum.
         with open(config.gpg.image_master, 'rb') as fp:
             checksum = hashlib.md5(fp.read()).digest()
         # The next state transition should get us a new image master.
-        next(state)
+        state.run_until('get_blacklist_2')
         # Now we have a new system image master key.
         with open(config.gpg.image_master, 'rb') as fp:
             self.assertNotEqual(checksum, hashlib.md5(fp.read()).digest())
         # Now the blacklist file's signature should be good.
-        next(state)
+        state.run_thru('get_blacklist_2')
         self.assertEqual(os.path.basename(state.blacklist), 'blacklist.tar.xz')
 
     @configuration
@@ -200,10 +264,11 @@ class TestState(unittest.TestCase):
         setup_keyring_txz(
             'spare.gpg', 'spare.gpg', dict(type='image-master'),
             os.path.join(self._serverdir, 'gpg', 'image-master.tar.xz'))
-        # Run the state machine once to grab the blacklist.  This should fail
-        # with a signature error (internally).  There will be no blacklist.
+        # Run the state machine long enough to grab the blacklist.  This
+        # should fail with a signature error (internally).  There will be no
+        # blacklist.
         state = State()
-        next(state)
+        state.run_thru('get_blacklist_1')
         self.assertIsNone(state.blacklist)
         # Just to provide that the system image master key is going to change,
         # let's calculate the current one's checksum.
@@ -211,7 +276,7 @@ class TestState(unittest.TestCase):
             checksum = hashlib.md5(fp.read()).digest()
         # The next state transition should get us a new image master, but its
         # signature is not good.
-        self.assertRaises(SignatureError, next, state)
+        self.assertRaises(SignatureError, state.run_until, 'get_blacklist_2')
         # And the old system image master key hasn't changed.
         with open(config.gpg.image_master, 'rb') as fp:
             self.assertEqual(checksum, hashlib.md5(fp.read()).digest())
@@ -226,11 +291,11 @@ class TestState(unittest.TestCase):
             'image-master.gpg', 'archive-master.gpg',
             dict(type='image-master'),
             os.path.join(self._serverdir, 'gpg', 'image-master.tar.xz'))
-        # Run the state machine once to get the blacklist.  This should
+        # Run the state machine long enough to get the blacklist.  This should
         # download the system image master key, which will be signed against
         # the archive master.  Prove that the image master doesn't exist yet.
         self.assertFalse(os.path.exists(config.gpg.image_master))
-        next(State())
+        State().run_thru('get_blacklist_1')
         # Now the image master key exists.
         self.assertTrue(os.path.exists(config.gpg.image_master))
 
@@ -248,11 +313,11 @@ class TestState(unittest.TestCase):
         setup_keyring_txz(
             'spare.gpg', 'spare.gpg', dict(type='blacklist'),
             os.path.join(self._serverdir, 'gpg', 'blacklist.tar.xz'))
-        # Run the state machine once to get the blacklist.  This should
+        # Run the state machine log enough to get the blacklist.  This should
         # download the system image master key, which will be signed against
         # the archive master.  Prove that the image master doesn't exist yet.
         self.assertFalse(os.path.exists(config.gpg.image_master))
-        next(State())
+        State().run_thru('get_blacklist_1')
         # Now the image master key exists.
         self.assertTrue(os.path.exists(config.gpg.image_master))
 
@@ -358,7 +423,7 @@ class _StateTestsBase(unittest.TestCase):
     def tearDown(self):
         self._stack.close()
 
-    def _setup_keyrings(self):
+    def _setup_keyrings(self, *, device_signing=True):
         # Only the archive-master key is pre-loaded.  All the other keys
         # are downloaded and there will be both a blacklist and device
         # keyring.  The four signed keyring tar.xz files and their
@@ -376,11 +441,12 @@ class _StateTestsBase(unittest.TestCase):
             'image-signing.gpg', 'image-master.gpg',
             dict(type='image-signing'),
             os.path.join(self._serverdir, 'gpg', 'image-signing.tar.xz'))
-        setup_keyring_txz(
-            'device-signing.gpg', 'image-signing.gpg',
-            dict(type='device-signing'),
-            os.path.join(self._serverdir, self.CHANNEL, self.DEVICE,
-                         'device-signing.tar.xz'))
+        if device_signing:
+            setup_keyring_txz(
+                'device-signing.gpg', 'image-signing.gpg',
+                dict(type='device-signing'),
+                os.path.join(self._serverdir, self.CHANNEL, self.DEVICE,
+                             'device-signing.tar.xz'))
 
 
 class TestRebooting(_StateTestsBase):
@@ -507,6 +573,78 @@ class TestRebooting(_StateTestsBase):
         self.assertTrue(got_reboot)
 
 
+class TestRebootingNoDeviceSigning(_StateTestsBase):
+    INDEX_FILE = 'index_13.json'
+    CHANNEL_FILE = 'channels_11.json'
+    CHANNEL = 'stable'
+    DEVICE = 'nexus7'
+
+    @configuration
+    def test_keyrings_copied_to_upgrader_paths_no_device_keyring(self):
+        # The following keyrings get copied to system paths that the upgrader
+        # consults:
+        # * blacklist.tar.xz{,.asc}      - data partition (if one exists)
+        # * image-master.tar.xz{,.asc}   - cache partition
+        # * image-signing.tar.xz{,.asc}  - cache partition
+        #
+        # In this test, there is no device signing keyring.
+        self._setup_keyrings(device_signing=False)
+        cache_dir = config.updater.cache_partition
+        data_dir = config.updater.data_partition
+        blacklist_path = os.path.join(data_dir, 'blacklist.tar.xz')
+        master_path = os.path.join(cache_dir, 'image-master.tar.xz')
+        signing_path = os.path.join(cache_dir, 'image-signing.tar.xz')
+        device_path = os.path.join(cache_dir, 'device-signing.tar.xz')
+        # None of the keyrings or .asc files are found yet.
+        self.assertFalse(os.path.exists(blacklist_path))
+        self.assertFalse(os.path.exists(master_path))
+        self.assertFalse(os.path.exists(signing_path))
+        self.assertFalse(os.path.exists(device_path))
+        self.assertFalse(os.path.exists(blacklist_path + '.asc'))
+        self.assertFalse(os.path.exists(master_path + '.asc'))
+        self.assertFalse(os.path.exists(signing_path + '.asc'))
+        self.assertFalse(os.path.exists(device_path + '.asc'))
+        # Be sure to resign the relevant files with the image-signing key.
+        index_path = os.path.join(
+            self._serverdir, self.CHANNEL, self.DEVICE, 'index.json')
+        sign(index_path, 'image-signing.gpg')
+        setup_index(self.INDEX_FILE, self._serverdir, 'image-signing.gpg')
+        # None of the data files are found yet.
+        for image in get_index('index_13.json').images:
+            for filerec in image.files:
+                path = os.path.join(cache_dir, os.path.basename(filerec.path))
+                asc = os.path.join(
+                    cache_dir, os.path.basename(filerec.signature))
+                self.assertFalse(os.path.exists(path))
+                self.assertFalse(os.path.exists(asc))
+        # Run the state machine enough times to download all the keyrings and
+        # data files, then to move the files into place just before a reboot
+        # is issued.  Steps preceded by * are steps that fail.
+        # *get blacklist/get master -> get channels/signing
+        # -> get device signing -> get index -> calculate winner
+        # -> download files -> move files
+        state = State()
+        state.run_thru('move_files')
+        # All of the keyrings and .asc files are found, except for the device
+        # singing keys.
+        self.assertTrue(os.path.exists(blacklist_path))
+        self.assertTrue(os.path.exists(master_path))
+        self.assertTrue(os.path.exists(signing_path))
+        self.assertFalse(os.path.exists(device_path))
+        self.assertTrue(os.path.exists(blacklist_path + '.asc'))
+        self.assertTrue(os.path.exists(master_path + '.asc'))
+        self.assertTrue(os.path.exists(signing_path + '.asc'))
+        self.assertFalse(os.path.exists(device_path + '.asc'))
+        # All of the data files are found.
+        for image in get_index('index_13.json').images:
+            for filerec in image.files:
+                path = os.path.join(cache_dir, os.path.basename(filerec.path))
+                asc = os.path.join(
+                    cache_dir, os.path.basename(filerec.signature))
+                self.assertTrue(os.path.exists(path))
+                self.assertTrue(os.path.exists(asc))
+
+
 class TestCommandFileFull(_StateTestsBase):
     INDEX_FILE = 'index_13.json'
     CHANNEL_FILE = 'channels_06.json'
